c--------------------------------------------------------------------------
c  find station list in cat file for kappa and make output suitable for
c  kappa.par
c  also count number of different stations
c
c   jh jan 2025
c--------------------------------------------------------------------------
c
c  For detail on parameters and variables names, see rea.inc
c
c
c  changes
c
c
c
      implicit none                       ! force delcaration of all variables
      include 'seidim.inc'                ! dimensions for rea block
      include 'seisan.inc'                ! dimensions for rea block
      include 'rea.inc'                   ! parameter common bliock

      character*80 data(5000)             ! s-file with data in text array

      logical all                         ! true: read all data, false: headers
      integer seiclen                     ! function
      integer code                        ! error return code
      integer nevent                      ! number of events in file
      character*10 keys                   !next choice key for routine findevin
      character*14 starttime,endtime      ! time interval to read
      character*80 basename               ! data base or file name
      character*80 infile                 ! input file or base name
      character*80 eventfile              ! single event file name
      character*5 stat(5000)              ! station code
      integer nstat                       ! number of different stations
      real max_dist                       ! maximum distance
      integer status,eventno,newmonth,fstart ! for routine findevin
      integer base                        ! type of input 0: seisan data base, 
                                          !     1: index file, 2: single file
      character*1 comp_selected(3)        ! components to use                                    
      integer i,k                         ! counter

      call get_seisan_def
c
c   open output file
c
      open(2,file='kappa_stat.out',status='unknown')

 1    continue
      write(6,*) 
     *'Enter components, one letter like Z, ZN or ZNE, enter for all'
      read(5,'(3a1)') comp_selected
      if(comp_selected(1).eq.' ') then
         comp_selected(1)='Z'
         comp_selected(2)='N'
         comp_selected(3)='E'
      endif

      If(comp_selected(1).ne.'Z'.and.
     *comp_selected(1).ne.'N'.and.comp_selected(1).ne.'E') then
         write(6,*)'Wrong component selection'
         goto 1
      endif
    
      
    
c
c   get input file name, or data base with time interval
c

      write(*,*) ' Event data input - select one:'
      write(*,*)
      WRITE(*,*) 
     *'    SEISAN default data base or                     :',
     *'Enter '
      write(*,*) 
     *'    Alternative data base, give 1-5 letter code or  :'  
      WRITE(*,*) 
     *'    Local index file, name must start with index or :'
      WRITE(*,*) 
     *'    Local data base, write ,, or                    :'
      WRITE(*,*) 
     *'    Filename for one file, min. 6 chars or with a . : '
      
      write(*,*)
      read(*,'(a)') infile
      basename=infile(1:80)
                       
      write(*,*)
C
c   check if this is a single multiple event file (base=2),
c   general data base (base=0) or  a local data base or 
c   index file (base=1)
c

      starttime=' '
      endtime=' '

      keys(1:4)='NEXT'    ! always use next event

c
c   initially assume a file
c
      base=2
c
c   a SEISAN 5 letter data base, blank is default data base, ',,'
c   is a local data base. the name cannot have a '.' and must
c   be less than 6 chars long
c
      if(seiclen(basename).lt.6.and.index(basename,'.').eq.0) then 
         base=0
      endif
c
c   case of index file or local data base
c
 
      if(basename(1:5).eq.'INDEX'.or.basename(1:5).eq.'index'.
     *     or.basename(1:2).eq.',,') then
        base=1
      endif
                              
c
c  get time interval for a seisan data base, no time interval used for
c  index file or local data base
c
      if(base.eq.0) then
         write(*,'('' Start Time           (YYYYMMDDHHMMSS): '',$)')
         read(*,'(a14)') starttime
         write(*,'('' End Time, enter is to end of month:    '',$)')
         read(*,'(a14)') endtime
         write(*,*)
      endif
c
c   open access for the the relevant input option
c

      write(6,*) 'Maxumum distance'
      read(5,*) max_dist

c
c  a file
c
      if(base.eq.2) then
         open(1,file=infile,status='old',err=5)
         goto 6
 5       continue
         write(6,*)' Input file not found'
         stop
 6       continue
      endif


      all=.true.                  ! read all parameters
      nevent=0                    ! initialize counter

c
c-----------------------------------------------------------------
c  Loop to read events start here
c-----------------------------------------------------------------
c

  50  continue

c
c   read from relevant input type
c
      if(base.lt.2) then     ! data base  or index file event
         call findevin
     *   (basename,starttime,endtime,keys,0,eventno,
     *   eventfile,fstart,newmonth,status)
c
c   check if end of time interval or errors, then
c   stop
C     
         if(status.gt.0) then
            write(6,*)' STOP WITH STATUS=',status
            goto 1000  ! stop
         endif
C
C   open data base input single event file
c

         open(1,file=eventfile,status='old',err=7)
         goto 8
 7       continue
         write(6,*)' Input file not found: ',eventfile
         stop
 8       continue
      endif

c
c   read all parameters for one event from file unit 1
c
      call rea_event_in(1,all,data,code)

c
c   close file if from data base since only one event
c
      if(base.lt.2) close(1)
c
c   check if end of file (code=1), if so jump out of loop
c
      if(code.eq.1) goto 1000
c
      nevent=nevent+1               ! count events
c
c   write the whole first header line
c
      write(6,'(a)') data(1)(1:79)
c
c   initialize
c
      if(nevent.eq.1) then
         nstat=1
         stat(1)=rea_stat(1)
      endif

c
c   find how many different stations
c

      do i=1,rea_nstat
         do k=1,nstat
            if(rea_stat(i).eq.stat(k)) goto 10 
         enddo
         if(rea_dist(i).le.max_dist) then
            nstat=nstat+1
            stat(nstat)=rea_stat(i)
         endif
 10      continue
      enddo

c
c   get next event
c
      goto 50
c
c     end of file or data base
c
 1000 continue
c
c    write in format for kappa.par
c


      do i=1,nstat
        do k=1,3
         if(comp_selected(k).eq.'Z')  write(2,'(a5,4x,a1)') stat(i),'Z'
         if(comp_selected(k).eq.'N')  write(2,'(a5,4x,a1)') stat(i),'N'
         if(comp_selected(k).eq.'E')  write(2,'(a5,4x,a1)') stat(i),'E'
        enddo
      enddo

      write(6,*)            ! blank line
      close(2)              ! close output file
      write(6,*) 'Number of different stations',nstat
      write(6,*) 'Number of events in input file or data base', nevent
      write(6,*) 'Output file name is kappa_stat.out'

      stop
      end
