c--------------------------------------------------------------------------
c  plot travel times made by lo
c--------------------------------------------------------------------------c
c
c     only phase lines with residual and distance will be selected
c

c
c changes:
c
c dec 28 2010 jh: gfortran on pc: remove winplot, implicit none, remove hctype, check
c                 units and computertype
c feb 22 2010 jh: size from color.def
c feb 17 2014 jh: read seisan.def, commnented out line 128, seems not
c                 to be used
c 2015.06.03 pv : moved line 9 continue, due to compiler warning on GOTO
c 2020 10 02 jh ; also plot g-phases
c 2024 10 02 jh : -do not list blank phases
c                 -fix probelm to show many stats at one point with click
c                 -possible to select only one phase
c                 -possible to use many events
c                 -do not recalcualate hypocenter
c                 -give origin with click on one observation
c                 -do not use BAZ phases
c                 -do not check for event types
c                 -do not use phases with weight 4 and station 
c                      with on phase with weight 9
c
c 2025 01 02 jh:  more output, plot resisual vs time, residul vs distance, 
c                 select a single station, histogram
c 
c
      implicit none                       ! force delcaration of all variables
      include 'seidim.inc'                ! dimensions for rea block
      include 'rea.inc'                   ! parameter common block
      include 'version.inc'
      include 'libsei.inc'

      include 'seiplot.inc'               ! seisan graphics
      character*1 cha(10)                 ! dummy for xy_plot

      character*120 txt(100)              ! messages
      character*120 txt120
      character*80 title                  ! title for plot
      character*80 answer                 ! input text
      character*30 xtext,ytext            ! axis title
      character*80 data(max_data)         ! s-file with data in text array
      character*80 infile                 ! input file
      character*8 phase(max_data)         ! phase name    
      character*8 pha                     ! phase name wanted 
      character*5 stat_selected           ! station wanted
      character*5  stat(max_data)         ! stations with p
      real         time(max_data)         ! abs arrival time
      real         res(max_data)          ! travel time residual
      real         dist(max_data)         ! distance from earthquake
      character*5  mag(max_data)          ! magnitude
      real         timecalp(max_data),timecals(max_data)      
                                          ! calculated first times
      real         distcal(max_data)
      character*8  sel_phase(max_data)    ! ------------------------
      character*5  sel_stat(max_data)     ! same as data above for
      real         sel_time(max_data)     ! selected stations
      real         sel_res(max_data)      ! selected stations
      real         sel_dist(max_data)     ! ------------------------
      character*5  sel_mag(max_data)      ! magnitudes
      real sel_year_time(max_data)        ! ----------time as yyyy.xxxx like 2002.3333
      character*20 sel_date_time(max_data)    ! ----------  
      
      character*20 date_time(max_data)     
      real year_time(max_data)            ! time as yyyy.xxxx like 2002.3333
      real corr, rms                      ! correlation coefficient and rms
      real velocity                       ! calculated velocity from slope
      real xm,ym                          ! cursor input
      real xc,yc                          ! scaling factors
      real*8 ot                           ! origin time
      real mindist,maxdist,xdist          ! distance range
      real maxtime
      integer ndist                       ! number of distances to calculate phases
      integer nphase                      ! number of phases in input
      integer nphase_selected             ! number of phaases after distance selection
      integer nevent                      ! number of events
      real a,b                            ! for lsq relation
      integer res_his(43)                 ! for residual histogram
      integer nres_out                    ! number of residuals outside range -2 to 2
      real x(1000),xx(8000),delta         ! help variabels for histogram
      real y(1000),yy(8000)               ! ----------------------------
      integer nhist                       ! number of points in histogram
      

      logical all                         ! true: read all data, false: headers
      logical use_residual                ! if true, plot residuals
      logical use_distance                ! if true, plot residuals vs distance
      logical use_traveltime              ! true if plot of distance vs traveltime times
      logical use_histogram               ! true if plotting hsitogram
      integer code                        ! error return code
      integer seiclen
      integer il                          ! line counter
      integer i,k,l,n,j,m                 ! counter
      real dist1,dist2                    ! distance range

      dist1=0.
      dist2=99999.
      nevent=0

      call get_seisan_def

      use_residual=.false.
      use_distance=.false.
      use_traveltime=.true. ! the default
      use_histogram=.false.
      stat_selected=' '



c
c   check if input via argumants
c
      call get_arguments(n,infile)
c
 9    continue
      if(n.lt.1) then
c
c   get input file name, check if exist
c
         write(6,*) 'Give input file'
         read(5,'(a)') infile
      endif

      open(1,file=infile,status='old',err=10)

      goto 11
 10   continue
      write(6,*)' No such input file'
      goto 9
 11   continue

      all=.true.                  ! read all parameters

      write(6,*)'Only plot one phase, give name, else enter'
      read(5,'(a)') pha
c
c   open output files
c

      open(2,file='ttplot.out',status='unknown')  
      open(3,file='ttplot_dist_trtime.out',status='unknown')
      open(4,file='ttplot_time_res.out',status='unknown')
      open(8,file='ttplot_dist_res.out',status='unknown')
      open(7,file='ttplot_histogram.out',status='unknown')  
      mindist=9999.
      maxdist=0.
      maxtime=0.
      k=1

c----------------------------------------------------------
c   event read loop, back here to read next event
c----------------------------------------------------------

 15   continue
c
c   read all parameters for one event from file unit 1
c
      call rea_event_in(1,all,data,code)
c
c   check if end of file (code=1), if so terminate program
c
      if(code.eq.1) goto 16
c
c   write the whole first header line
c
      write(6,'(a)') data(1)(1:79)
      nevent=nevent+1

c
c compute abs origin time
c
      call timsec(hyp_year(1),hyp_month(1),hyp_day(1),
     &   hyp_hour(1),hyp_min(1),hyp_sec(1),ot)
c
c   weight out all phases for a station with a weight 9 
c
      do i=1,rea_nphase
         if(rea_weight_in(i).eq.'9') then
            do m=1,rea_nphase
                if(rea_stat(i).eq.rea_stat(m)) rea_weight_in(m)='4'
            enddo
         endif
      enddo
c
c   store phases
c

      do i=1,rea_nphase
        if (rea_phase(i)(1:2).ne.'AM'.and.
     *      rea_phase(i)(1:1).ne.'E'.and. rea_phase(i)(1:1).ne.'I'.
     *      and.rea_phase(i)(1:1).ne.' '.and.
     *      rea_phase(i)(1:3).ne.'BAZ'.and.
     *      rea_phase(i).ne.'SPEC'.and.
     *      rea_weight_in(i).ne.'4') then
c
c   check if distance and residual
c
         if (rea_dist(i).ne.-999.0.and.rea_res(i).ne.-999.0) then
c
c   phase
c
           if(pha.ne.' '.and.pha.eq.rea_phase(i).or.pha.eq.' ') then
              call timsec(hyp_year(1),hyp_month(1),hyp_day(1),
     &        rea_hour(i),rea_min(i),rea_sec(i),rea_abs_time(i))
              time(k)=rea_abs_time(i)-ot
              stat(k)=rea_stat(i)
              dist(k)=rea_dist(i)
              res(k)=rea_res(i)
              phase(k)=rea_phase(i)
c
c   find first magnitude ml
c
              do l=1,6
                 if(hyp_mag_type(l,1).eq.'L') then
                    write(mag(k),'(f5.1)') hyp_mag(l,1)
                    goto 12
                 else
                    mag(k)=' '
                 endif
              enddo
 12           continue
                    
              year_time(k)= 
     *        hyp_year(1)+float(hyp_month(1))/12.0+float(hyp_day(1))/365.0
              date_time(k)=data(1)(1:20)
c
c   find distance range
c
              if (dist(k).gt.maxdist) maxdist=dist(k)
              if (dist(k).lt.mindist) mindist=dist(k)
              if (time(k).gt.maxtime) maxtime=time(k)
              k=k+1
           endif
         else
           write(6,'(a,a,a,a)')
     *    'No residual or distance for station and phase: ',
     *     rea_stat(i),rea_phase(i),data(1)(1:20)
           write(6,*)'Data skipped ********************'
         endif
        endif
      enddo
c
c   get next event
c
      goto 15

c------------------------------------------------------------
c   end of reading events
c------------------------------------------------------------

 16   continue
      nphase=k-1

      write(6,*)'Total number of events in input      ',nevent
      write(6,*)'Total number of valid phases in input',nphase
      

      if(nphase.eq.0) then
         write(6,*)'No phases'
         stop
      endif
c
c   if more than one event, maybe use only one station
c
      if(nevent.gt.1) then
         write(6,*)
     *   'If use only one station, give station code, else enter'
         read(5,'(a)')stat_selected
         if(stat_selected.ne.' ') then
           l=0
           do i=1,nphase  
              if (stat(i).eq.stat_selected) then
                 l=l+1
                 dist(l)=dist(i)
                 phase(l)=phase(i)
                 stat(l)=stat(i)
                 time(l)=time(i)
                 res(l)=res(i)
                 mag(l)=mag(i)
                 year_time(l)=year_time(i)
                 date_time(l)=date_time(i)
              endif
           enddo
           nphase=l
           write(6,*)
     *    'Total number of valid phases for given station',nphase
        endif
      endif
   

c
c  set defaults for output on screen and one hardcopy file
c
      open(65,file='ttplot.eps',status='unknown')
      plotunit=65
      plotoption=1
      wsize=60
      call get_window_size
      if(size_ttplot.gt.0) wsize=size_ttplot ! from color.def
c
c
c   open plotter
c
c
c   set some postscipt scalings
c
      write(65,*) ' 1.0 0.55 scale'

c
c   open display
c
      call open_display
c
c--------------------------------------------------------------------
c   back here for new distance limits
c--------------------------------------------------------------------

 49   continue

c
c  distance selection from total data set
c
      l=0
      do i=1,nphase  
        if (dist(i).ge.dist1.and.dist(i).le.dist2) then
          l=l+1
          sel_dist(l)=dist(i)
          sel_phase(l)=phase(i)
          sel_stat(l)=stat(i)
          sel_time(l)=time(i)
          sel_res(l)=res(i)
          sel_mag(l)=mag(i)
          sel_year_time(l)=year_time(i)
          sel_date_time(l)=date_time(i)
        endif
      enddo

      nphase_selected=l
      write(6,*) 'Number of phases after distance selection ',
     *nphase_selected
      if(nphase_selected.eq.0) then
         write(6,*) 'No data, will stop'
         stop
      endif
c
c   values for histogram
c

c
c   convert res to a number between 1 and 41 for residual histogram
c

      do i=1,43
        res_his(i)=0
      enddo
      nres_out=0
      do i=1,nphase_selected
         k=sel_res(i)*10+21
         if(k.ge.1.and.k.lt.42)then
            res_his(k)=res_his(k)+1
            nres_out=nres_out+1
         endif
      enddo
      do i=1,41
         x(i)=(i-21)/10.0
         y(i)=res_his(i)
      enddo

      write(6,*)
     *'Number of residuals inside and outside range -2 to 2 ',
     * nres_out,nphase_selected-nres_out
c
c--------------------------------------------------------------------
c   back here to start plot again
c--------------------------------------------------------------------

 50   continue

      call clear_display
c
c-----------------------------------------------------------------------
c   plot travel time vs distance, the default
c-----------------------------------------------------------------------
c
      if(use_traveltime) then
c
c   plot text
c
          txt(1)='Select station by'
          txt(2)='clicking near symbol'
          call xmessage(txt,2,25,720.0,650.0)
          title=' '
c
c   help text
c
              txt(1)='r: replot   d: dist. selection'//
     &    '   R: res. vs time  D: res. vs dist.  h: histogram'//
     *    '    q: quit '
       call xmessage(txt,1,seiclen(txt(1)),100.0,720.0)

c
c   if only one phase, plot name
c

c
c   if only one station, plot name
c
          if(stat_selected.ne.' ') then
             txt(1)=stat_selected 
             call xmessage(txt,1,5,110.0,655.0)
          endif   

          if(pha.ne.' ') then
             txt(1)=pha
             call xmessage(txt,1,8,155.0,655.0)
          endif 
          
          xtext=' Distance-km '
          ytext=' Travel time-sec '

         call xy_plot
     *   (1,nphase_selected,sel_dist,sel_time,title,xtext,ytext,
     *   600.0,600.0,100.0,100.0,1,1,20.0,
     *   0,cha,i,dist,time)
c
c
c   make lsq and plot line if only one phase

         if(pha.ne.' ') then   
            call lsqlin(nphase_selected,sel_dist,sel_time,a,b,corr,rms)
            if(b.ne.0.0) then
               velocity=1.0/b
            else 
               velocity=999.99
            endif
   
c
c   plot line with seisan routine given equation for line and size of coordinate system
c
            call xy_plot_line(b,a,100.0,100.0)

            write(txt120,'(a,a,a,f7.3,a,f7.3,a,f7.3,a,f4.2,a,f5.2)')
     *      'Least squares relation for ',pha,
     *      ' time = ',b,' * dist + ',a,'  velocity=',velocity,
     *      '  corr=',corr,'  rms=',rms

c
c   plot lsq relation
c
            call xmessage(txt120,1,seiclen(txt120),100.0,740.0)
          endif

c
c   if only one phase, skip plotting calculated travel time curves for crustal phases 
c

        if(pha.ne.' ') goto 25

c
c---------------------------------------------------------------------------
c    calculate and plot theoretical  travel times unless only one phase
c---------------------------------------------------------------------------
c

      ndist=100
c
c   calculate first arrivals
c
      call traveltimes(hyp_model(1),mindist,maxdist,ndist,hyp_depth(1),
     *   timecalp,timecals,distcal,' ')
c
c   first arrival P
c

        k=0
        do j=1,ndist
          if (distcal(j).le.maxdist.and.
     *        timecalp(j).le.maxtime) k=j
        enddo

        call xy_plot
     *    (1,k,distcal,timecalp,title,xtext,ytext,
     *    0.,600.0,100.0,100.0,2,1,20.0,
     *    0,cha,i,dist,time)
c
c   first arrival s
c
        k=0
        do j=1,ndist
          if (distcal(j).le.maxdist.and.
     *        timecals(j).le.maxtime) k=j
        enddo
        call xy_plot
     *    (1,k,distcal,timecals,title,xtext,ytext,
     *    0.,600.0,100.0,100.0,2,1,20.0,
     *    0,cha,i,dist,time)

c
c   calculate g-phases
c
      call traveltimes(hyp_model(1),mindist,maxdist,ndist,hyp_depth(1),
     *   timecalp,timecals,distcal,'G')
c
c  plot Pg
c
        k=0
        do j=1,ndist
          if (distcal(j).le.maxdist.and.
     *        timecalp(j).le.maxtime) k=j
        enddo

        call xy_plot
     *    (1,k,distcal,timecalp,title,xtext,ytext,
     *    0.,600.0,100.0,100.0,2,1,20.0,
     *    0,cha,i,dist,time)
c
c   plot Sg
c
        k=0
        do j=1,ndist
          if (distcal(j).le.maxdist.and.
     *        timecals(j).le.maxtime) k=j
        enddo
        call xy_plot
     *    (1,k,distcal,timecals,title,xtext,ytext,
     *    0.,600.0,100.0,100.0,2,1,20.0,
     *    0,cha,i,dist,time)

 25     continue      ! get here if not plotting travel time curves

c
c   call up cursor so plots remains
c

          call xy_plot_input(100.0,100.0,cha(1),xm,ym,xc,yc)

          use_traveltime=.false.

          if(cha(1).eq.'r') then
               use_traveltime=.true.
               goto 50
          endif
          if(cha(1).eq.'R') then
               use_residual=.true.
               goto 50
          endif
          if(cha(1).eq.'d') then
             use_traveltime=.true.
          endif
          if(cha(1).eq.'D') then
               use_distance=.true.
               goto 50
          endif
          if(cha(1).eq.'h') then 
              use_histogram=.true.
              goto 50
          endif
          use_traveltime=.true.

        endif   ! section travel time

c---------------------------------------------------------------------
c   plot histogram
c---------------------------------------------------------------------

      if(use_histogram) then
c
c   generate points of histogram, assume input x is equidistant
c
         k=1
         nhist=41
c
c   distance between columns
c
         delta=x(2)-x(1)
         do i=1,nhist-1
            xx(k)=x(i)
            yy(k)=0.0
            k=k+1
            xx(k)=x(i)
            yy(k)=y(i)
            k=k+1
            xx(k)=x(i+1)
            yy(k)=y(i)
            k=k+1
            xx(k)=x(i+1)
            yy(k)=0.0
            k=k+1
         enddo
         xx(k)=x(nhist)
         yy(k)=y(nhist)
         k=k+1
         xx(k)=x(nhist)+delta
         yy(k)=y(nhist)
         k=k+1
         xx(k)=xx(k-1)
         yy(k)=0.0
         k=k+1
         xx(k)=xx(1)
         yy(k)=yy(1)
c
c   shift all columns
c
         do i=1,k
           xx(i)=xx(i)-delta/2.0
         enddo
c
c   plot 
c
         call clear_display

         title=' '
         xtext='Travel time residuals'
         ytext='Number '
c
c    help text
c
          txt(1)='r: replot   d: dist. selection'//
     &    '   R: res. vs time  D: res. vs dist.  h: histogram'//
     *    '    q: quit '
              call xmessage(txt,1,seiclen(txt(1)),100.0,720.0)

c
c   if only one station, plot name
c
          if(stat_selected.ne.' ') then
             txt(1)=stat_selected 
             call xmessage(txt,1,5,110.0,655.0)
          endif      
c
c   if only one phase, plot name
c
          if(pha.ne.' ') then
            txt(1)=pha
            call xmessage(txt,1,8,155.0,655.0)
          endif
             

          call xy_plot
     *    (1,k,xx,yy,title,xtext,ytext,
     *    600.0,600.0,100.0,100.0,2,1,20.0,
     *    0,cha,i,xc,yc)


          call xy_plot_input(100.0,100.0,cha(1),xm,ym,xc,yc)

          use_histogram=.false.

          if(cha(1).eq.'r') then
               use_traveltime=.true.
               goto 50
          endif
          if(cha(1).eq.'R') then
               use_residual=.true.
               goto 50
          endif
          if(cha(1).eq.'d') then
             use_traveltime=.true.
          endif
          if(cha(1).eq.'D') then
               use_distance=.true.
               goto 50
          endif
          if(cha(1).eq.'h') then 
              use_histogram=.true.
              goto 50
          endif
          use_histogram=.true.
         
      endif
c
c-------------------------------------------------------------------------------
c   plot residuals vs time
c-------------------------------------------------------------------------------
c

      if(use_residual) then
         if(nevent.eq.1) then
            txt(1)='Residual vs time not possible for one event'
            call xmessage(txt,1,seiclen(txt(1)),120.0,620.0)
            use_residual=.false.
            use_traveltime=.true.
            goto 50
         endif

         xtext=' Time in years '
         ytext=' Travel time residual '
c
c   plot points
c
         call clear_display

c
c    help text
c

         txt(1)='r: replot   d: dist. selection'//
     &    '   R: res. vs time  D: res. vs dist.  h: histogram'//
     *    '    q: quit '
             
         call xmessage(txt,1,seiclen(txt(1)),100.0,720.0)
c
c   plot text
c  
         txt(1)='Select station by'
         txt(2)='clicking near symbol'
         call xmessage(txt,2,25,720.0,650.0)

c
c   if only one station, plot name
c
         if(stat_selected.ne.' ') then
            txt(1)=stat_selected 
            call xmessage(txt,1,5,110.0,655.0)
         endif      
c
c   if only one phase, plot name
c
         if(pha.ne.' ') then
           txt(1)=pha
           call xmessage(txt,1,8,155.0,655.0)
         endif
       
         call xy_plot
     *   (1,nphase_selected,sel_year_time,sel_res,
     *   title,xtext,ytext,
     *   600.0,600.0,100.0,100.0,1,1,20.0,
     *   0,cha,i,dist,time)

         call xy_plot_input(100.0,100.0,cha(1),xm,ym,xc,yc)

         use_residual=.false.

         if(cha(1).eq.'r') then
             use_traveltime=.true.
             goto 50
         endif
         if(cha(1).eq.'R') then
            use_residual=.true.
            goto 50
         endif
         if(cha(1).eq.'d') then
             use_traveltime=.true.
         endif
         if(cha(1).eq.'D') then
             use_distance=.true.
             goto 50
         endif
         if(cha(1).eq.'h') then 
            use_histogram=.true.
            goto 50
         endif

         use_residual=.true.


      endif
c
c-------------------------------------------------------------------------------
c   plot residuals vs distance
c-------------------------------------------------------------------------------
c
      if(use_distance) then

         xtext=' Distance-km '
         ytext=' Travel time residual '
c
c   plot points
c
         call clear_display

c
c    help text
c
         txt(1)='r: replot   d: dist. selection'//
     &    '   R: res. vs time  D: res. vs dist.  h: histogram'//
     *    '    q: quit '
              call xmessage(txt,1,seiclen(txt(1)),100.0,720.0)
c
c   plot text
c  
         txt(1)='Select station by'
         txt(2)='clicking near symbol'
         call xmessage(txt,2,25,720.0,650.0)

c
c   if only one station, plot name
c
         if(stat_selected.ne.' ') then
            txt(1)=stat_selected
            call xmessage(txt,1,5,110.0,655.0)
         endif      
      
c
c   if only one phase, plot name
c
         if(pha.ne.' ') then
            txt(1)=pha
            call xmessage(txt,1,8,155.0,655.0)
         endif
       
         call xy_plot
     *   (1,nphase_selected,sel_dist,sel_res,title,xtext,ytext,
     *   600.0,600.0,100.0,100.0,1,1,20.0,
     *   0,cha,i,dist,time)
         call xy_plot_input(100.0,100.0,cha(1),xm,ym,xc,yc)

         use_distance=.false.

         if(cha(1).eq.'r') then
              use_traveltime=.true.
              goto 50
         endif
         if(cha(1).eq.'R') then
              use_residual=.true.
              goto 50
         endif
         if(cha(1).eq.'d') then
            use_traveltime=.true.
         endif
         if(cha(1).eq.'D') then
              use_distance=.true.
              goto 50
         endif
         if(cha(1).eq.'h') then 
             use_histogram=.true.
             goto 50
         endif
         use_distance=.true.

      endif


c----------------------------------------------------------------
c    check if distance selection,  replot or residual
c----------------------------------------------------------------

      if (cha(1).eq.'d') then
         txt(1)='Give distance range (d1,d2) '
         call oneline(txt(1),seiclen(txt(1)),answer,20,150.,500.)
         call sei get values(2,answer,code)
         dist1=ARRAY$(1)
         dist2=ARRAY$(2)
         use_residual=.false.
         use_distance=.false.
         use_traveltime=.true.
         goto 49
      endif

      if(cha(1).eq.'q') goto 1000
 
c--------------------------------------------------------
c   find corresponding station(s) from a click on screen
c--------------------------------------------------------

 70    continue    ! loop for station selection

c   time vs distance

 
      if(use_traveltime) then
         j=1
         do i=1,nphase_selected
            if(abs(xm-sel_dist(i))*xc.lt.10.0.and.   ! distance in pixels
     *      abs(ym-sel_time(i))*yc.lt.8.0) then
               txt(1)='STAT  PHASE      DIST   TIME    RES'
               j=j+1
               write(txt(j),
     *         '(a5,1x,a8,1x,f6.1,1x,f6.1,1x,f6.2)')
     *         sel_stat(i),sel_phase(i),sel_dist(i),
     *         sel_time(i),sel_res(i)
               j=j+1
               txt(j)=' '
               txt(j)(1:19)=sel_date_time(i)(2:20)
            endif
        enddo
      endif
c
c  residuals vs time
c
      if(use_residual) then
         j=1
         do i=1,nphase_selected
            if(abs(xm-year_time(i))*xc.lt.10.0.and.   ! distance in pixels
     *         abs(ym-res(i))*yc.lt.8.0) then
               txt(1)='STAT  PHASE      DIST   TIME    RES'
               j=j+1
               write(txt(j),
     *         '(a5,1x,a8,1x,f6.1,1x,f6.1,1x,f6.2)')
     *         sel_stat(i),sel_phase(i),sel_dist(i),
     *         sel_time(i),sel_res(i)
               j=j+1
               txt(j)=' '
               txt(j)(1:19)=sel_date_time(i)(2:20)
            endif
          enddo
        endif

c  residuals vs distance

       if(use_distance) then
          j=1
          do i=1,nphase_selected
             if(abs(xm-sel_dist(i))*xc.lt.10.0.and.   ! distance in pixels
     *          abs(ym-sel_res(i))*yc.lt.8.0) then
                txt(1)='STAT  PHASE      DIST   TIME    RES'
                j=j+1
                write(txt(j),
     *          '(a5,1x,a8,1x,f6.1,1x,f6.1,1x,f6.2)')
     *          sel_stat(i),sel_phase(i),sel_dist(i),
     *          sel_time(i),sel_res(i)
                j=j+1
                txt(j)=' '
                txt(j)(1:19)=sel_date_time(i)(2:20)
             endif
           enddo
       endif

       if(use_traveltime.or.use_distance.or.use_residual) then
          if(j.gt.1) then
             do i=j+1,10
               txt(i)=' '
             enddo
             call xmessage(txt,10,40,720.0,500.0)
          else
             txt(1)='Too far from symbol,'
             txt(2)='try again'
             do i=3,10
               txt(i)=' '
             enddo
             call xmessage(txt,10,40,720.0,500.0)
          endif
       endif


       if(cha(1).ne.'R'.and.cha(1).ne.'r'.and.cha(1).ne.'D'.and.
     *    cha(1).ne.'h'.and.cha(1).ne.'d'.and.cha(1).ne.'q') then  ! click for station

          call xy_plot_input(100.0,100.0,cha(1),xm,ym,xc,yc)
          goto 70
       else
          use_traveltime=.false.
          use_residual=.false.
          use_distance=.false.
          use_histogram=.false.

          if(cha(1).eq.'r') then
               use_traveltime=.true.
               goto 50
          endif
          if(cha(1).eq.'R') then
               use_residual=.true.
               goto 50
          endif
          if(cha(1).eq.'d') then
             goto 50
c             use_traveltime=.true.
          endif
          if(cha(1).eq.'D') then
               use_distance=.true.
               goto 50
          endif
          if(cha(1).eq.'h') then 
              use_histogram=.true.
              goto 50
          endif
          if(cha(1).eq.'q') goto 1000
       endif



 1000 continue

      write(6,*)
      write(6,*)'Writing output files of selected values ............'
      write(6,*)
c
c   write all phases to output and distance travel time in separate file
c
      write(2,'(a)')
     *'STAT  Phase   Distance Tr time     Res       Time'//
     *'        Date and time    Ml'

      do i=1,nphase_selected
           write(2,'(a5,1x,a8,1x,f7.1,1x,f7.1,1x,f7.2,1x,
     *     f10.4,1x,a20,1x,a5)')
     &     sel_stat(i),sel_phase(i),sel_dist(i),sel_time(i),sel_res(i),
     *     sel_year_time(i),sel_date_time(i), sel_mag(i)

           write(4,'(f10.4,1x,f10.2)')
     *     sel_year_time(i),sel_res(i)
           write(3,'(2f8.2)') sel_dist(i),sel_time(i) 

           write(8,'(f10.1,f10.2)') sel_dist(i),sel_res(i)
      enddo

c
c   write histogram
c          
      do i=1,41
         write(7,'(f7.2,i6)')(i-21)/10.0,res_his(i)
         x(i)=(i-21)/10.0
         y(i)=res_his(i)
      enddo

c 
c   close postscript
c
          call close_post
c
c   close output plot file
c
          close(65)
c
c   close display and back to alpha screen
c
         write(6,*)
         write(6,*)'Plot file is ttplot.eps'

 30   call clear_to_alpha

c
c     end of file
c

c
      write(6,*)            ! blank line
      write(6,*)'Output ttplot.out has all info'
      write(6,*)'Output ttplot_dist_trtime has travel time vs distance' 
      write(6,*)'Output ttplot_dist_res.out has residual vs distance'
      write(6,*)'Output ttplot_time_res.out has residual vs time'
      write(6,*)'Output ttplot_histogram.out has histogram of residuals'


      stop
      end

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      subroutine xy_plot_input(x0,y0,c,x,y,xc,yc)
c
c  reads positions from the screen much like the main routine
c  all scaling values from routine xy_plot via common
c  x0,y0 are lower left hand corner of plot
c
c  x0,y0 : same input as main routine
c  x,y   : output in scaled units
c  c     : character pressed
c  xc    : xscale
c
       implicit none
       character*1 c
       integer ichar
       reaL xc,yc   ! same as x and yscale
       real x1,x,y1,y   ! help variables
       real xfirst,yfirst,x0,y0,xscale,yscale,ymin,ymax,xmin,xmax ! see xy_plot
       common/xy_plot_common/xfirst,yfirst,xscale,yscale,ymin,
     *                       ymax,xmin,xmax
c
c   call up cursor
c

   1        continue
            call xscursr(ichar,x1,y1)
            if (char(ichar).eq.'#') goto 1
            c=char(ichar)
            x=(x1-x0)/xscale+xfirst
            y=(y1-y0)/yscale+yfirst
            xc=xscale
            yc=yscale
c
      return
      end

      subroutine traveltimes(model,mindist,maxdist,ndist,depth,
     *   timep,times,distance,prmd)

c**********************************************************************
c subroutine to calculate travel times for given model and distance range
c modified from program ttlayer
c***********************************************************************
     
      implicit none
      include 'hypparm.inc'
      character*1 model
      real mindist,maxdist,depth,delta_dist
      integer ndist
      real timep(*),times(*),distance(*)
      character*80 infile,outfile  
      character*80 indat,testdat
      character*1 reff,ucase,prmd,prm2
      character*4 stat
      character*8 phsid
      real vpp(nlayer)
      logical exist
      character*60 top_directory
      character*1 dchar
      real rearth
      integer seiclen
      real pi,degtorad,testj,xnear,xfar,dist,tmin,delta,tpp,ann
      integer i,j,nmoho,nconrad,iulst,nd,iflag,minflag,iustat

      pi=3.141593
      degtorad=pi/180.
      rearth=6371.                         

c
c get directory structure
c
      call topdir(top_directory)
      call dir_char(dchar)         ! dirctory delimiter character
      infile='STATION0.HYP'
      if (model.ne.' ') write(infile(8:8),'(a1)') model

c
c find station file
c
      inquire(file=infile, exist=exist)
      if (.not.exist) then

         infile = top_directory(1:seiclen(top_directory)) // dchar // 
     &         'DAT' // dchar //
     &         infile(1:seiclen(infile))

         inquire(file=infile, exist=exist)
         if (.not.exist) then
           write(*,*) 'station file does not exist !'
           stop
         endif
      endif
      open(9,file=infile)
      outfile = 'ttlayer.out'
      open(10,file=outfile)
      iustat=9         
      open(14,file='out')
c    set test parameter defaults
        call settest(test)

c    read test parameter changes and station data
c    changed 4/94 to read character string and check for ")" so reset test(1) is OK
        stat='xxxx'
        j=-1
        
        do while (stat.ne. '    '.and.j.ne.0)
         read(iustat,'(a80)')testdat
         if(testdat(14:14).eq.')')then
          read(testdat,'(a4,t12,i2,t16,f9.4)')stat,j,testj
         elseif(testdat(13:13).eq.')')then
          read(testdat,'(a4,t12,i1,t15,f9.4)')stat,j,testj
         else
          read(testdat,'(a4)')stat
          j=-1
         endif
         if(j.gt.0)then
          test(j)=testj
         endif
        end do

      indat='XXXXXXXX'
      do while (indat(1:8).ne.'        ')
       read(9,'(a80)')indat
      end do

c read the velocity model
        i=1
        iustat=9
        
c    reff is used to specify the moho layer for PN calculation
5       read(iustat,101,end=99)v(i),d(i),vs(i),reff
101     format(3f7.3,a1)
        if(ucase(reff).eq.'N')nmoho=i

c 4/94: added nconrad variable
        if(ucase(reff).eq.'B')nconrad=i
        
        if(v(i).eq.0.0)go to 6
        i=i+1
        go to 5

c    nl is the number of layers in the velocity model
6       nl=i-1

c    read in trial depth and vp/vs ratio
        read(iustat,'(3f5.0,f5.2)',end=99)ztr,xnear,xfar,pos

c    if vs(1)=0 then set vs(i)=v(i)/pos
        if(vs(1).eq.0.0)then
          do i=1,nl
            vs(i)=v(i)/pos
          end do
        endif

c  store thicknesses in parm
        do i=1,nl-1
          parm(nl+i)=d(i+1)-d(i)

c    change 10/93: add maximum elevation to upper layer thickness
c    if test(40)=0.0 Need this because dtdx2 origin is always at maxelv
c          if(i.eq.1.and.test(40).eq.0.0)then
c            parm(nl+i)=d(i+1)-d(i)
c          endif
        end do
        do i=1,nl
          parm(i)=v(i)
        end do
c       nn=2*nl-1

      minflag=int(test(63))      
        
c    xs(1), xs(2) are the station long. and lat.
      x0(1)=0.0
      x0(2)=0.0
      x0(3)=0.0
      
c      prmd=' '                        
      prm2=' '

      iulst=10
      nd=100

      xh(2)=0.0
      xh(3)=depth

c
c write some info
c
      delta_dist=(maxdist-mindist)/float(ndist)
      dist=mindist
      ndist=0    ! now used as counter
      do while (dist.le.maxdist) 
       ndist=ndist+1
       do i=1,nl
         parm(i)=v(i)
       end do
       xh(1)=dist/rearth
c
c p time
c
       call dtdx2(xh,x0,prmd,nmoho,nconrad,iulst,tmin,
     &  dx,delta,ann,iflag,phsid)
       if(iflag.eq.0)tmin=0.0
       tpp=tmin
       do i=1,nl
          parm(i)=vs(i)
          vpp(i)=v(i)
       end do
c
c s time
c
       call dtdx2(xh,x0,prmd,nmoho,nconrad,iulst,tmin,
     &  dx,delta,ann,iflag,phsid)
       if(iflag.eq.0)tmin=0.0
       
       do i=1,nl
         v(i)=vs(i)
       enddo
       timep(ndist)=tpp
       times(ndist)=tmin
       distance(ndist)=dist
       write(10,'(5f12.3)')dist,tpp,tmin
       dist=dist+delta_dist
       do i=1,nl
         v(i)=vpp(i)
       enddo
      end do
      
99    continue
      close(9)
      close(10)
      return
      end
      

